@@ -4,11 +4,11 @@
  *	  search code for postgres btrees.
  *
  *
- * Portions Copyright (c) 1996-2001, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2002, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  * IDENTIFICATION
- *	  $Header: /home/projects/pgsql/cvsroot/pgsql/src/backend/access/nbtree/nbtsearch.c,v 1.66 2001/03/23 04:49:51 momjian Exp $
+ *	  $Header: /cvsroot/pgsql-server/src/backend/access/nbtree/nbtsearch.c,v 1.72.2.1 2003/07/29 22:18:48 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -19,7 +19,7 @@
 #include "access/nbtree.h"
 
 
-static RetrieveIndexResult _bt_endpoint(IndexScanDesc scan, ScanDirection dir);
+static bool _bt_endpoint(IndexScanDesc scan, ScanDirection dir);
 
 
 /*
@@ -61,6 +61,13 @@ _bt_search(Relation rel, int keysz, ScanKey scankey,
 		BlockNumber par_blkno;
 		BTStack		new_stack;
 
+		/*
+		 * Race -- the page we just grabbed may have split since we read
+		 * its pointer in the parent (or metapage).  If it has, we may need
+		 * to move right to its new sibling.  Do that.
+		 */
+		*bufP = _bt_moveright(rel, *bufP, keysz, scankey, BT_READ);
+
 		/* if this is a leaf page, we're done */
 		page = BufferGetPage(*bufP);
 		opaque = (BTPageOpaque) PageGetSpecialPointer(page);
@@ -94,16 +101,9 @@ _bt_search(Relation rel, int keysz, ScanKey scankey,
 		new_stack->bts_parent = stack_in;
 
 		/* drop the read lock on the parent page, acquire one on the child */
-		_bt_relbuf(rel, *bufP, BT_READ);
+		_bt_relbuf(rel, *bufP);
 		*bufP = _bt_getbuf(rel, blkno, BT_READ);
 
-		/*
-		 * Race -- the page we just grabbed may have split since we read
-		 * its pointer in the parent.  If it has, we may need to move
-		 * right to its new sibling.  Do that.
-		 */
-		*bufP = _bt_moveright(rel, *bufP, keysz, scankey, BT_READ);
-
 		/* okay, all set to move down a level */
 		stack_in = new_stack;
 	}
@@ -155,7 +155,7 @@ _bt_moveright(Relation rel,
 		/* step right one page */
 		BlockNumber rblkno = opaque->btpo_next;
 
-		_bt_relbuf(rel, buf, access);
+		_bt_relbuf(rel, buf);
 		buf = _bt_getbuf(rel, rblkno, access);
 		page = BufferGetPage(buf);
 		opaque = (BTPageOpaque) PageGetSpecialPointer(page);
@@ -357,11 +357,11 @@ _bt_compare(Relation rel,
  *
  *		On entry, we have a valid currentItemData in the scan, and a
  *		read lock and pin count on the page that contains that item.
- *		We return the next item in the scan, or NULL if no more.
+ *		We return the next item in the scan, or false if no more.
  *		On successful exit, the page containing the new item is locked
- *		and pinned; on NULL exit, no lock or pin is held.
+ *		and pinned; on failure exit, no lock or pin is held.
  */
-RetrieveIndexResult
+bool
 _bt_next(IndexScanDesc scan, ScanDirection dir)
 {
 	Relation	rel;
@@ -374,7 +374,7 @@ _bt_next(IndexScanDesc scan, ScanDirection dir)
 	BTScanOpaque so;
 	bool		continuescan;
 
-	rel = scan->relation;
+	rel = scan->indexRelation;
 	so = (BTScanOpaque) scan->opaque;
 	current = &(scan->currentItemData);
 
@@ -386,7 +386,7 @@ _bt_next(IndexScanDesc scan, ScanDirection dir)
 	{
 		/* step one tuple in the appropriate direction */
 		if (!_bt_step(scan, &buf, dir))
-			return (RetrieveIndexResult) NULL;
+			return false;
 
 		/* current is the next candidate tuple to return */
 		offnum = ItemPointerGetOffsetNumber(current);
@@ -397,7 +397,8 @@ _bt_next(IndexScanDesc scan, ScanDirection dir)
 		if (_bt_checkkeys(scan, itup, dir, &continuescan))
 		{
 			/* tuple passes all scan key conditions, so return it */
-			return FormRetrieveIndexResult(current, &(itup->t_tid));
+			scan->xs_ctup.t_self = itup->t_tid;
+			return true;
 		}
 
 		/* This tuple doesn't pass, but there might be more that do */
@@ -406,25 +407,26 @@ _bt_next(IndexScanDesc scan, ScanDirection dir)
 	/* No more items, so close down the current-item info */
 	ItemPointerSetInvalid(current);
 	so->btso_curbuf = InvalidBuffer;
-	_bt_relbuf(rel, buf, BT_READ);
+	_bt_relbuf(rel, buf);
 
-	return (RetrieveIndexResult) NULL;
+	return false;
 }
 
 /*
  *	_bt_first() -- Find the first item in a scan.
  *
  *		We need to be clever about the type of scan, the operation it's
- *		performing, and the tree ordering.	We return the RetrieveIndexResult
- *		of the first item in the tree that satisfies the qualification
+ *		performing, and the tree ordering.	We find the
+ *		first item in the tree that satisfies the qualification
  *		associated with the scan descriptor.  On exit, the page containing
  *		the current index tuple is read locked and pinned, and the scan's
  *		opaque data entry is updated to include the buffer.
  */
-RetrieveIndexResult
+bool
 _bt_first(IndexScanDesc scan, ScanDirection dir)
 {
-	Relation	rel;
+	Relation	rel = scan->indexRelation;
+	BTScanOpaque so = (BTScanOpaque) scan->opaque;
 	Buffer		buf;
 	Page		page;
 	BTStack		stack;
@@ -434,10 +436,9 @@ _bt_first(IndexScanDesc scan, ScanDirection dir)
 	ItemPointer current;
 	BlockNumber blkno;
 	StrategyNumber strat;
-	RetrieveIndexResult res;
-	RegProcedure proc;
+	bool		res;
 	int32		result;
-	BTScanOpaque so;
+	bool		scanFromEnd;
 	bool		continuescan;
 	ScanKey		scankeys = NULL;
 	int			keysCount = 0;
@@ -446,26 +447,23 @@ _bt_first(IndexScanDesc scan, ScanDirection dir)
 				j;
 	StrategyNumber strat_total;
 
-	rel = scan->relation;
-	so = (BTScanOpaque) scan->opaque;
-
 	/*
 	 * Order the scan keys in our canonical fashion and eliminate any
 	 * redundant keys.
 	 */
-	_bt_orderkeys(rel, so);
+	_bt_orderkeys(scan);
 
 	/*
 	 * Quit now if _bt_orderkeys() discovered that the scan keys can never
 	 * be satisfied (eg, x == 1 AND x > 2).
 	 */
 	if (!so->qual_ok)
-		return (RetrieveIndexResult) NULL;
+		return false;
 
 	/*
 	 * Examine the scan keys to discover where we need to start the scan.
 	 */
-	scan->scanFromEnd = false;
+	scanFromEnd = false;
 	strat_total = BTEqualStrategyNumber;
 	if (so->numberOfKeys > 0)
 	{
@@ -512,13 +510,13 @@ _bt_first(IndexScanDesc scan, ScanDirection dir)
 			}
 		}
 		if (keysCount == 0)
-			scan->scanFromEnd = true;
+			scanFromEnd = true;
 	}
 	else
-		scan->scanFromEnd = true;
+		scanFromEnd = true;
 
 	/* if we just need to walk down one edge of the tree, do that */
-	if (scan->scanFromEnd)
+	if (scanFromEnd)
 	{
 		if (nKeyIs)
 			pfree(nKeyIs);
@@ -532,6 +530,8 @@ _bt_first(IndexScanDesc scan, ScanDirection dir)
 	scankeys = (ScanKey) palloc(keysCount * sizeof(ScanKeyData));
 	for (i = 0; i < keysCount; i++)
 	{
+		FmgrInfo   *procinfo;
+
 		j = nKeyIs[i];
 
 		/*
@@ -543,11 +543,15 @@ _bt_first(IndexScanDesc scan, ScanDirection dir)
 			pfree(nKeyIs);
 			pfree(scankeys);
 			elog(ERROR, "_bt_first: btree doesn't support is(not)null, yet");
-			return ((RetrieveIndexResult) NULL);
+			return false;
 		}
-		proc = index_getprocid(rel, i + 1, BTORDER_PROC);
-		ScanKeyEntryInitialize(scankeys + i, so->keyData[j].sk_flags,
-							   i + 1, proc, so->keyData[j].sk_argument);
+		procinfo = index_getprocinfo(rel, i + 1, BTORDER_PROC);
+		ScanKeyEntryInitializeWithInfo(scankeys + i,
+									   so->keyData[j].sk_flags,
+									   i + 1,
+									   procinfo,
+									   CurrentMemoryContext,
+									   so->keyData[j].sk_argument);
 	}
 	if (nKeyIs)
 		pfree(nKeyIs);
@@ -569,7 +573,7 @@ _bt_first(IndexScanDesc scan, ScanDirection dir)
 		ItemPointerSetInvalid(current);
 		so->btso_curbuf = InvalidBuffer;
 		pfree(scankeys);
-		return (RetrieveIndexResult) NULL;
+		return false;
 	}
 
 	/* remember which buffer we have pinned */
@@ -584,16 +588,16 @@ _bt_first(IndexScanDesc scan, ScanDirection dir)
 
 	/*
 	 * At this point we are positioned at the first item >= scan key, or
-	 * possibly at the end of a page on which all the existing items are 
-	 * greater than the scan key and we know that everything on later pages
-	 * is less than or equal to scan key.
-     *
+	 * possibly at the end of a page on which all the existing items are
+	 * less than the scan key and we know that everything on later
+	 * pages is greater than or equal to scan key.
+	 *
 	 * We could step forward in the latter case, but that'd be a waste of
 	 * time if we want to scan backwards.  So, it's now time to examine
 	 * the scan strategy to find the exact place to start the scan.
 	 *
 	 * Note: if _bt_step fails (meaning we fell off the end of the index in
-	 * one direction or the other), we either return NULL (no matches) or
+	 * one direction or the other), we either return false (no matches) or
 	 * call _bt_endpoint() to set up a scan starting at that index
 	 * endpoint, as appropriate for the desired scan type.
 	 *
@@ -610,7 +614,7 @@ _bt_first(IndexScanDesc scan, ScanDirection dir)
 			if (!_bt_step(scan, &buf, BackwardScanDirection))
 			{
 				pfree(scankeys);
-				return (RetrieveIndexResult) NULL;
+				return false;
 			}
 			break;
 
@@ -644,7 +648,7 @@ _bt_first(IndexScanDesc scan, ScanDirection dir)
 			if (!_bt_step(scan, &buf, BackwardScanDirection))
 			{
 				pfree(scankeys);
-				return (RetrieveIndexResult) NULL;
+				return false;
 			}
 			break;
 
@@ -659,7 +663,7 @@ _bt_first(IndexScanDesc scan, ScanDirection dir)
 				if (!_bt_step(scan, &buf, ForwardScanDirection))
 				{
 					pfree(scankeys);
-					return (RetrieveIndexResult) NULL;
+					return false;
 				}
 				offnum = ItemPointerGetOffsetNumber(current);
 				page = BufferGetPage(buf);
@@ -701,7 +705,7 @@ _bt_first(IndexScanDesc scan, ScanDirection dir)
 				if (!_bt_step(scan, &buf, ForwardScanDirection))
 				{
 					pfree(scankeys);
-					return (RetrieveIndexResult) NULL;
+					return false;
 				}
 			}
 			break;
@@ -717,7 +721,7 @@ _bt_first(IndexScanDesc scan, ScanDirection dir)
 				if (!_bt_step(scan, &buf, ForwardScanDirection))
 				{
 					pfree(scankeys);
-					return (RetrieveIndexResult) NULL;
+					return false;
 				}
 				offnum = ItemPointerGetOffsetNumber(current);
 				page = BufferGetPage(buf);
@@ -728,7 +732,7 @@ _bt_first(IndexScanDesc scan, ScanDirection dir)
 				if (!_bt_step(scan, &buf, ForwardScanDirection))
 				{
 					pfree(scankeys);
-					return (RetrieveIndexResult) NULL;
+					return false;
 				}
 				offnum = ItemPointerGetOffsetNumber(current);
 				page = BufferGetPage(buf);
@@ -747,7 +751,8 @@ _bt_first(IndexScanDesc scan, ScanDirection dir)
 	if (_bt_checkkeys(scan, itup, dir, &continuescan))
 	{
 		/* yes, return it */
-		res = FormRetrieveIndexResult(current, &(itup->t_tid));
+		scan->xs_ctup.t_self = itup->t_tid;
+		res = true;
 	}
 	else if (continuescan)
 	{
@@ -760,8 +765,8 @@ _bt_first(IndexScanDesc scan, ScanDirection dir)
 nomatches:
 		ItemPointerSetInvalid(current);
 		so->btso_curbuf = InvalidBuffer;
-		_bt_relbuf(rel, buf, BT_READ);
-		res = (RetrieveIndexResult) NULL;
+		_bt_relbuf(rel, buf);
+		res = false;
 	}
 
 	pfree(scankeys);
@@ -783,7 +788,7 @@ nomatches:
 bool
 _bt_step(IndexScanDesc scan, Buffer *bufP, ScanDirection dir)
 {
-	Relation	rel = scan->relation;
+	Relation	rel = scan->indexRelation;
 	ItemPointer current = &(scan->currentItemData);
 	BTScanOpaque so = (BTScanOpaque) scan->opaque;
 	Page		page;
@@ -815,14 +820,14 @@ _bt_step(IndexScanDesc scan, Buffer *bufP, ScanDirection dir)
 				/* if we're at end of scan, release the buffer and return */
 				if (P_RIGHTMOST(opaque))
 				{
-					_bt_relbuf(rel, *bufP, BT_READ);
+					_bt_relbuf(rel, *bufP);
 					ItemPointerSetInvalid(current);
 					*bufP = so->btso_curbuf = InvalidBuffer;
 					return false;
 				}
 				/* step right one page */
 				blkno = opaque->btpo_next;
-				_bt_relbuf(rel, *bufP, BT_READ);
+				_bt_relbuf(rel, *bufP);
 				*bufP = _bt_getbuf(rel, blkno, BT_READ);
 				page = BufferGetPage(*bufP);
 				opaque = (BTPageOpaque) PageGetSpecialPointer(page);
@@ -846,7 +851,7 @@ _bt_step(IndexScanDesc scan, Buffer *bufP, ScanDirection dir)
 				/* if we're at end of scan, release the buffer and return */
 				if (P_LEFTMOST(opaque))
 				{
-					_bt_relbuf(rel, *bufP, BT_READ);
+					_bt_relbuf(rel, *bufP);
 					ItemPointerSetInvalid(current);
 					*bufP = so->btso_curbuf = InvalidBuffer;
 					return false;
@@ -854,7 +859,7 @@ _bt_step(IndexScanDesc scan, Buffer *bufP, ScanDirection dir)
 				/* step left */
 				obknum = BufferGetBlockNumber(*bufP);
 				blkno = opaque->btpo_prev;
-				_bt_relbuf(rel, *bufP, BT_READ);
+				_bt_relbuf(rel, *bufP);
 				*bufP = _bt_getbuf(rel, blkno, BT_READ);
 				page = BufferGetPage(*bufP);
 				opaque = (BTPageOpaque) PageGetSpecialPointer(page);
@@ -868,7 +873,7 @@ _bt_step(IndexScanDesc scan, Buffer *bufP, ScanDirection dir)
 				while (opaque->btpo_next != obknum)
 				{
 					blkno = opaque->btpo_next;
-					_bt_relbuf(rel, *bufP, BT_READ);
+					_bt_relbuf(rel, *bufP);
 					*bufP = _bt_getbuf(rel, blkno, BT_READ);
 					page = BufferGetPage(*bufP);
 					opaque = (BTPageOpaque) PageGetSpecialPointer(page);
@@ -897,7 +902,7 @@ _bt_step(IndexScanDesc scan, Buffer *bufP, ScanDirection dir)
  * that the scan must start at the beginning or end of the index (for
  * a forward or backward scan respectively).
  */
-static RetrieveIndexResult
+static bool
 _bt_endpoint(IndexScanDesc scan, ScanDirection dir)
 {
 	Relation	rel;
@@ -912,10 +917,10 @@ _bt_endpoint(IndexScanDesc scan, ScanDirection dir)
 	BTItem		btitem;
 	IndexTuple	itup;
 	BTScanOpaque so;
-	RetrieveIndexResult res;
+	bool		res;
 	bool		continuescan;
 
-	rel = scan->relation;
+	rel = scan->indexRelation;
 	current = &(scan->currentItemData);
 	so = (BTScanOpaque) scan->opaque;
 
@@ -931,7 +936,7 @@ _bt_endpoint(IndexScanDesc scan, ScanDirection dir)
 		/* empty index... */
 		ItemPointerSetInvalid(current);
 		so->btso_curbuf = InvalidBuffer;
-		return (RetrieveIndexResult) NULL;
+		return false;
 	}
 
 	blkno = BufferGetBlockNumber(buf);
@@ -952,7 +957,7 @@ _bt_endpoint(IndexScanDesc scan, ScanDirection dir)
 		itup = &(btitem->bti_itup);
 		blkno = ItemPointerGetBlockNumber(&(itup->t_tid));
 
-		_bt_relbuf(rel, buf, BT_READ);
+		_bt_relbuf(rel, buf);
 		buf = _bt_getbuf(rel, blkno, BT_READ);
 
 		page = BufferGetPage(buf);
@@ -968,7 +973,7 @@ _bt_endpoint(IndexScanDesc scan, ScanDirection dir)
 			do
 			{
 				blkno = opaque->btpo_next;
-				_bt_relbuf(rel, buf, BT_READ);
+				_bt_relbuf(rel, buf);
 				buf = _bt_getbuf(rel, blkno, BT_READ);
 				page = BufferGetPage(buf);
 				opaque = (BTPageOpaque) PageGetSpecialPointer(page);
@@ -1011,7 +1016,7 @@ _bt_endpoint(IndexScanDesc scan, ScanDirection dir)
 	if (start > maxoff)
 	{
 		if (!_bt_step(scan, &buf, dir))
-			return (RetrieveIndexResult) NULL;
+			return false;
 		start = ItemPointerGetOffsetNumber(current);
 		page = BufferGetPage(buf);
 	}
@@ -1023,7 +1028,8 @@ _bt_endpoint(IndexScanDesc scan, ScanDirection dir)
 	if (_bt_checkkeys(scan, itup, dir, &continuescan))
 	{
 		/* yes, return it */
-		res = FormRetrieveIndexResult(current, &(itup->t_tid));
+		scan->xs_ctup.t_self = itup->t_tid;
+		res = true;
 	}
 	else if (continuescan)
 	{
@@ -1035,8 +1041,8 @@ _bt_endpoint(IndexScanDesc scan, ScanDirection dir)
 		/* no tuples in the index match this scan key */
 		ItemPointerSetInvalid(current);
 		so->btso_curbuf = InvalidBuffer;
-		_bt_relbuf(rel, buf, BT_READ);
-		res = (RetrieveIndexResult) NULL;
+		_bt_relbuf(rel, buf);
+		res = false;
 	}
 
 	return res;
